function [Pc,c_mu,priorA] = TFBS_DF_Likelihood_Double(S,PP,M,B,offset)

% [Pc,c_mu,priorA] = TFBS_DF_Likelihood_Double(S,PP,M,B,offset)
%
% This function implements a multiple transcription factor binding site 
% (TFBS) prediction method for a whole promoter sequence and both strands 
% of the DNA using a likelihood based method. This function also has an 
% option for general data fusion via a positional prior.
%
% INPUT:
% S       - A row vector of size 1-by-(L+offset) consisting of integers 
%           between 1 and 4. L is the length of the sequence and offset 
%           specifies the number of base pairs that are used to extend the 
%           sequences from the beginning/left. Thus, S(offset+1:offset+L) 
%           encodes the actual promoter sequence. Base pairs are coded 
%           using integers as follows: A = 1, C = 2, G = 3, and T = 4.
% PP      - "Positional prior" specifying a (prior) probability of having a
%           binding site at each location of the sequence. PP has the same
%           size as S. PP is primarily meant to be used for general data 
%           fusion. For example, PP can be constructed from different 
%           additional data sources, such as estimated conservation 
%           probabilities (e.g. PhastCons), regulatory potential scores 
%           (e.g. ESPERR), predicted nucleosome occupancy probabilities, 
%           predicted CpG islands, etc. It is typically advantageous to 
%           properly scale/smooth the above mentioned "raw" probabilities/
%           scores before use.
% M       - A 1-by-m cell vector specifying m different (normalized) motif 
%           models (product multinomials). M{k} is a 4-by-l(k) matrix 
%           specifying the k:th motif model. M{k}(i,j) defines the 
%           probability of observing a nucleotide i at position j in the 
%           motif model k.
% B       - A (4^k)-by-4 matrix specifying a k:th order Markovian 
%           background model. B(i,j) defines the probability of observing a
%           nucleotide j given that the previous k nucleotides are i (as 
%           encoded into an integer) in the background B. An example: if
%           k=3, then i=1 corresponds to AAA, i=2 to AAC, i=3 to AAG, i=4
%           to AAT, i=5 to ACA, ..., i=64 to TTT.
% offset  - A parameter specifying the offset of the promoter start site. 
%           If offset>0, then base pairs S(1:offset) are not considered as 
%           possible locations of motifs but can be used for a higher-
%           order (Markovian) background model.
%
% OUTPUT:
% Pc      - Likelihood of S containing c (c=0,1,...,floor(L/l)) motifs
%           instances.
% c_mu    - The expected number of motifs in sequence S.
% priorA  - Logarithm of the prior over multiple motif configurations for 
%           each fixed number of motifs, which is assumed to be uniform and
%           inverse proportional to the number different multiple motif 
%           configuraions.

% 08/11/2006 by Harri Lhdesmki


[Pc1,c_mu1,priorA] = TFBS_DF_Likelihood2(S,PP,M,B,offset);
[Pcr,c_mur,priorAr] = TFBS_DF_Likelihood2(oppositestrand(S),...
    PP(end:-1:1),M,B,offset);

Pc = zeros(1,length(Pc1));
for i=1:length(Pcr)
    Pc(i) = sum(Pc1(1:i).*Pcr(i:-1:1));
end

c_mu = sum(Pc.*[0:length(Pc)-1]);
